#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Standard includes.
import sys
import os
import pkg_resources

# Trac includes.
from trac import __version__ as VERSION
from trac.env import Environment
from trac.config import Option
from trac.mimeview import Context
from trac.admin.console import TracAdmin, TRAC_VERSION
from trac.util.datefmt import format_datetime
from trac.util.text import to_unicode, print_table, pretty_size

# TracDownloads includes.
from tracdownloads.api import DownloadsApi

class TracDownloadsAdmin(TracAdmin):

    path = Option('downloads', 'path', '/var/lib/trac/downloads',
      'Directory to store uploaded downloads.')

    def __init__(self, envdir = None):
        TracAdmin.__init__(self, envdir)

    def all_docs(self):
        docs = TracAdmin.all_docs()
        docs.extend(self._help_downloads)
        return docs

    _help_downloads = [("downloads list", "Show downloads uploaded to "
      "environment"), ("downloads add", ""), ("downloads remove id"
      " <download_id>", "Remove download with ID <download_id>"),
      ("downloads remove file <download_filename>", "Remove download with"
      " filename <download_filename>")]

    def do_downloads(self, line):
        arg = self.arg_tokenize(line)
        if arg[0]  == 'list':
            self._do_downloads_list()
        elif arg[0] == 'add':
            self._do_downloads_add(*(arg[1:]))
        elif arg[0] == 'remove':
            self._do_downloads_remove(*(arg[1:]))
        else:
            self.do_help('downloads')

    def _do_downloads_list(self):
        # Get downloads API component.
        api = DownloadsApi(self.env_open())

        # Create context.
        context = Context('downloads-core')
        db = self.env_open().get_db_cnx()
        context.cursor = db.cursor()

        # Print uploded download
        downloads = api.get_downloads(context)
        print_table([(download['id'], download['file'], pretty_size(
          download['size']), format_datetime(download['time']),
          download['component'],  download['version'],
          download['architecture']['name'], download['platform']['name'],
          download['type']['name']) for download in downloads], ['ID',
          'Filename', 'Size', 'Uploaded', 'Component', 'Version',
          'Architecture', 'Platform', 'Type'])

    def _do_downloads_add(self, *arguments):
        print "_do_downloads_add(%s)" % (arguments,)

    def _do_downloads_remove(self, *arguments):
        # Check correct number of arguments.
        if len(arguments) != 2:
            return self.do_help('downloads')

        # Get downloads API component.
        api = DownloadsApi(self.env_open())

        # Create context.
        env = self.env_open()
        context = Context('downloads-core')
        db = env.get_db_cnx()
        context.cursor = db.cursor()

        # Delete download by ID or filename.
        if arguments[0] == 'id':
            download = api.get_download(context, arguments[1])

            #try:
            env.log.debug(self.path)
            env.log.debug(path)
            api.delete_download(context, download['id'])
            path = os.path.join(self.path, to_unicode(download['id']))
            filepath = os.path.join(path, download['file'])
            path = os.path.normpath(path)
            filepath = os.path.normpath(filepath)
            os.remove(filepath)
            os.rmdir(path)
            #except Exception, error:
            #    env.log.error(error)

            # Commit changes in DB.
            db.commit()

        elif arguments[1] == 'file':
            # TODO: Not implemented.
            pass
        else:
            self.do_help('downloads')

def run(args = None):
    if args is None:
        args = sys.argv[1:]
    admin = TracDownloadsAdmin()
    if len(args) > 0:
        if args[0] in ('-h', '--help', 'help'):
            return admin.onecmd('help')
        elif args[0] in ('-v', '--version'):
            printout(os.path.basename(sys.argv[0]), TRAC_VERSION)
        else:
            env_path = os.path.abspath(args[0])
            try:
                unicode(env_path, 'ascii')
            except UnicodeDecodeError:
                printerr(_("non-ascii environment path '%(path)s' not "
                           "supported.", path=env_path))
                sys.exit(2)
            admin.env_set(env_path)
            if len(args) > 1:
                s_args = ' '.join(["'%s'" % c for c in args[2:]])
                command = args[1] + ' ' +s_args
                return admin.onecmd(command)
            else:
                while True:
                    try:
                        admin.run()
                    except KeyboardInterrupt:
                        admin.do_quit('')
    else:
        return admin.onecmd("help")

if __name__ == '__main__':
    pkg_resources.require('Trac==%s' % VERSION)
    sys.exit(run())
